@@ -2,11 +2,6 @@ package WebService::Raygun::Messenger;
 
 use Mouse;
 
-use LWP::UserAgent;
-use URI;
-use Mozilla::CA;
-use JSON;
-
 #use Smart::Comments;
 
 =head1 NAME
@@ -29,6 +24,27 @@ Send a request to raygun.io.
 
 =cut
 
+=head2 api_key
+
+Your raygun.io API key. By default, this will be whatever is in the
+RAYGUN_API_KEY environment variable.
+
+=cut
+
+use LWP::UserAgent;
+use URI;
+use Mozilla::CA;
+use JSON;
+use Mouse::Util::TypeConstraints;
+
+subtype 'RaygunMessage' => as 'Object' => where {
+    $_->isa('WebService::Raygun::Message');
+};
+
+coerce 'RaygunMessage' => from 'HashRef' => via {
+    return WebService::Raygun::Message->new( %{$_} );
+};
+
 has api_key => (
     is       => 'rw',
     isa      => 'Str',
@@ -52,6 +68,12 @@ has user_agent => (
     },
 );
 
+has message => (
+    is     => 'rw',
+    isa    => 'RaygunMessage',
+    coerce => 1,
+);
+
 =head2 fire_raygun
 
 Send data to api.raygun.io/entries via a POST request.
@@ -59,27 +81,27 @@ Send data to api.raygun.io/entries via a POST request.
 =cut
 
 sub fire_raygun {
-    my ( $self, $message ) = @_;
+    my $self    = shift;
+    my $message = $self->message;
     my $uri     = $self->api_endpoint;
     my $ua      = $self->user_agent;
     my $api_key = $self->api_key;
-    my $json = JSON->new->allow_nonref;
-    my $jsoned =$json->pretty->encode($message);
-    my $req = HTTP::Request->new(POST => $uri);
-    $req->header('Content-Type' => 'application/json');
-    $req->header('X-ApiKey' => $api_key);
+    my $json    = JSON->new->allow_nonref;
+    my $jsoned  = $json->pretty->encode( $message->prepare_raygun );
+    ### json : $jsoned
+    my $req     = HTTP::Request->new( POST => $uri );
+    $req->header( 'Content-Type' => 'application/json' );
+    $req->header( 'X-ApiKey'     => $api_key );
     $req->content($jsoned);
     ### json message : $jsoned;
     my $response = $ua->request($req);
 
     #my $response =
-      #$ua->post( $uri, 'X-ApiKey' => $api_key, Content => [ $jsoned ] );
+    #$ua->post( $uri, 'X-ApiKey' => $api_key, Content => [ $jsoned ] );
     return $response;
 
 }
 
-
-
 1;
 
 __END__
