@@ -10,6 +10,7 @@ use Test::Deep;    # (); # uncomment to stop prototype errors
 use Test::Exception;
 
 use HTTP::Request;
+
 #use Smart::Comments;
 
 sub prep001_messenger_available : Test(startup => 2) {
@@ -22,45 +23,48 @@ sub prep001_messenger_available : Test(startup => 2) {
 }
 
 sub t0010_raygun_http_403_response : Test(2) {
-    my $self = shift;
-    my $messenger = WebService::Raygun::Messenger->new( api_key => '' );
+    my $self      = shift;
+    my $messenger = WebService::Raygun::Messenger->new(
+        api_key => '',
+        message => test_message()
+    );
     my $result;
     lives_ok {
-        $result = $messenger->fire_raygun( {} );
+        $result = $messenger->fire_raygun();
     }
     'Called Raygun.io';
     cmp_ok( $result->code, '==', 403, 'Expect a "Bad Request" error.' );
 }
 
-sub t0020_raygun_http_400_response : Test(2) {
+sub t0020_raygun_http_ok_response : Test(2) {
     my $self    = shift;
     my $api_key = $ENV{RAYGUN_API_KEY};
     if ( not defined $api_key ) {
         $self->SKIP_ALL('No API key for Raygun.io. No point in continuing.');
     }
     $self->{api_key} = $api_key;
-    my $messenger = WebService::Raygun::Messenger->new( api_key => $self->{api_key} );
+    my $messenger = WebService::Raygun::Messenger->new(
+        api_key => $self->{api_key},
+        message => test_message()
+    );
     my $result;
     lives_ok {
         $result = $messenger->fire_raygun( {} );
     }
     'Called Raygun.io';
 
-    cmp_ok( $result->code, '==', 400, 'Expect a "Bad Request" error.' );
-
+    cmp_ok( $result->code, '<', 400, 'Expect a "Bad Request" error.' );
 }
 
-sub t0030_raygun_http_ok : Test(2) {
-    my $self    = shift;
-    my $api_key = $self->{api_key};
-    my $message = WebService::Raygun::Message->new(
-        user => 'null@null.com',
+sub test_message {
+    my $message = {
+        user   => 'null@null.com',
         client => {
             name      => 'something',
             version   => 2,
             clientUrl => 'www.null.com'
         },
-        error       => {
+        error => {
             stack_trace => [ { line_number => 34 } ]
         },
         environment => {
@@ -73,20 +77,8 @@ sub t0030_raygun_http_ok : Test(2) {
             POST => 'https://www.null.com',
             [ 'Content-Type' => 'text/html', ]
         ),
-
-    );
-
-    my $prepare_raygun = $message->prepare_raygun;
-    my $messenger = WebService::Raygun::Messenger->new( api_key => $self->{api_key} );
-    my $response;
-    lives_ok {
-        $response = $messenger->fire_raygun($prepare_raygun);
-    }
-    'Request worked ok';
-    ### response : $response
-    cmp_ok( $response->code, '<', 400, 'Expect OK response.' );
-
-
+    };
+    return $message;
 }
 
 1;
